use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/Perl/Critic/Policy/ValuesAndExpressions/PreventSQLInjection.pm',
    't/00-load.t',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/30-policy.t',
    't/35-default_themes.t',
    't/45-multi-docs.t',
    't/ValuesAndExpressions/PreventSQLInjection.run',
    't/files/test10.pl',
    't/files/test11.pl'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
