use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Alders
Alders'
Aubert
Charlie
Critic
Ferraz
Garrison
Guillaume
MERCHANTABILITY
MetaCPAN
Nelson
ORM
Olaf
PPI
Perl
Policy
PreventSQLInjection
ValuesAndExpressions
Victor
at
aubertg
cng
cpan
heredoc
lib
nelson
olaf
victor
